#version 120
#IFDEF shader4
#extension GL_EXT_gpu_shader4: enable
#ENDIF



#IFDEF threeComp
//threcomp needs index 0 to be 0
uniform vec3 normals[7];
#ELSE
uniform vec3 normals[6];
#ENDIF


#IMPORT data/shader/cube/cubeLightPerVertex.glsl

uniform vec3 shift;
uniform vec3 quadPosMark[6];
uniform mat4 v_inv;
const float tiling = 0.0625;
const float tilingH = 0.03;
const float adi = 0.00485;
const float oreOverlayStartingRow = 1.0;
uniform int animationTime;
uniform int allLight;



varying float textureIndex;
varying vec4 occlusion;
varying float layer;
varying vec3 lightDir;
varying vec4 tTexCoords;
varying vec3 viewDirection;
varying vec2 oreOverlayTexCoords;
varying vec4 vPos;
varying vec3 normal;
varying float extraAlphaFromVS;
varying float noLight;
varying vec3 normalVec;
varying vec3 camPos; // MOD ADDED

/*
void colorSide(float side){
	if(side == 0.0){
	//front
		light = vec4(1.0, 0.0, 0.0,   1.0);
	}else if(side == 1.0){
	//back
		light = vec4(0.0, 1.0, 0.0,   1.0);
	}else if(side == 2.0){
	//top
		//light = vec4(0.0, 0.0, 1.0,   1.0);
	}else if(side == 3.0){
	//bot
		//light = vec4(1.0, 1.0, 0.0,   1.0);
	}else if(side == 4.0){
	//right
		light = vec4(0.0, 0.0, 1.0,   1.0);
	}else if(side == 5.0){
	//left
		light = vec4(0.0, 1.0, 1.0,   1.0);
	}

}
*/
void main()
{

	#IFDEF shader4

	int indexInfo = int(gl_Vertex.x);
	float vIndex = float((indexInfo ) & 4095);

	float red =  float((indexInfo >> 12) & 15);

	float green =  float((indexInfo >> 16) & 15);

	float blue =  float((indexInfo >> 20) & 15);



	int info = int(gl_Vertex.y);

	float vertNumCodeE = float(info & 3);

	int sideId =  (info >> 2) & 7; // /4.0

	float layerE =  float((info >> 5) & 15);

	float typeE =  float((info >> 9) & 255) ;

	float hitPointsE =  float((info >> 17) & 7) ;

	float animatedE = float((info >> 20) & 1);

	float extFlag =  float(info >> 21);


	int normalIndex = sideId;
	vec3 qpm = quadPosMark[normalIndex];


	#ELSE


	float colorInfoX = gl_Vertex.x;


	float blue =  floor(colorInfoX / 1048576.0);
	colorInfoX -= blue * 1048576.0;

	float green =  floor(colorInfoX / 65536.0);
	colorInfoX -= green * 65536.0;

	float red =  floor(colorInfoX / 4096.0);
	colorInfoX -= red * 4096.0;

	float vIndex = floor(colorInfoX);


	float texInfoY = gl_Vertex.y;

	float extFlag =  floor(texInfoY / 2097152.0);
	texInfoY -= extFlag * 2097152.0;

	float animatedE =  floor(texInfoY / 1048576.0);
	texInfoY -= animatedE * 1048576.0;

	float hitPointsE =  floor(texInfoY / 131072.0) ;
	texInfoY -= hitPointsE * 131072.0;

	float typeE =  floor(texInfoY / 512.0) ;
	texInfoY -= typeE * 512.0;

	float layerE =  floor(texInfoY * 0.03125) ; // / 32.0
	texInfoY -= layerE * 32.0;

	float sideId =  (floor(texInfoY * 0.25)); // /4.0
	texInfoY -= sideId * 4.0;

	float vertNumCodeE = (floor(texInfoY));

	int normalIndex = int(sideId);
	vec3 qpm = quadPosMark[normalIndex];

	#ENDIF





	float oreOverlayE = 0;
	float occE = 0;

	#IFDEF threeComp

		float inside = 0.5;

		float posInfoZ = gl_Vertex.z;

		float zPos = floor(posInfoZ / 65536.0);
		posInfoZ = posInfoZ - (zPos * 65536.0);
		float yPos = floor(posInfoZ * 0.00390625); //divided by 256
		float xPos = ( posInfoZ - (yPos * 256));



		float sInfoW = gl_Vertex.w;


		occE =  floor(sInfoW / 65536.0) ; //32768.0, 65536.0, 131072.0, 262144.0

		sInfoW -= occE * 65536.0;

		oreOverlayE =  floor(sInfoW / 1024.0) ;

		sInfoW -= oreOverlayE * 1024.0;


		float normalModeE =  sInfoW;



		vec3 normalPos = normals[normalIndex+1];

		if(normalModeE == 0){
			normal = normalPos;
		}else{
			float nAf = (floor(normalModeE * 0.015625)); // div by 64.0
			normalModeE -= nAf*64.0;

			float nBf = (floor(normalModeE * 0.125)); // div by 8.0
			normalModeE -= nBf*8.0;

			float nCf = normalModeE;

			int nA = int(nAf);
			int nB = int(nBf);
			int nC = int(nCf);

			vec3 com = normals[nA]  + normals[nB] + normals[nC];
			normal = normalize(com);
		}
	#ELSE
		float inside = 0.5;
		vec3 normalPos = normals[normalIndex];
		normal = normalPos;

	#ENDIF


	float type = typeE;

	vec2 texCoords;

	float mVertNum = (vertNumCodeE) * 0.25;
	float mTex = (extFlag) * 0.25;



	if(allLight > 1){
		occlusion =  vec4(0.6,0.6,0.6, 0.7);
		noLight = 1.0;
	}else{
		occlusion = vec4(red * 0.066666, green * 0.066666, blue * 0.066666, occE * 0.066666); // / 15
		noLight = 0.0;
	}
	layer = layerE;



	type += animatedE * animationTime;



	float xIndex = mod(type, 16.0);
	float yIndex = floor(type * 0.0625 ); // / 16.0






	vec2 quad = vec2(
			mod(floor(mTex * (2.0)), 2.0),
			mod(floor(mTex * (4.0)), 2.0)
			);
	//either 0,0; 0,1; 1,1; 1,0




	 //8,8,8 = 8736
	float z = floor(vIndex * 0.00390625); //divided by 256
	vIndex -= z * 256.0;
	float y = floor(vIndex * 0.0625);
	vIndex -= y * 16.0;
	float x = vIndex;


	vec3 vertexPos = vec3(x-8.0, y-8.0, z-8.0);




	float xP = ((1.0 - abs(normalPos.x)) * -0.5) + mod(floor(mVertNum * qpm.x), 2.0);
	float yP = ((1.0 - abs(normalPos.y)) * -0.5) + mod(floor(mVertNum * qpm.y), 2.0);
	float zP = ((1.0 - abs(normalPos.z)) * -0.5) + mod(floor(mVertNum * qpm.z), 2.0);

	vertexPos += vec3(xP,yP,zP) + (normalPos * inside);




	vec2 adip = vec2(
	((1.0 -((quad.x)*adi)) + (abs(quad.x - 1.0)*adi)),
	((1.0 -((quad.y)*adi)) + (abs(quad.y - 1.0)*adi)));

	texCoords = vec2(quad.x *tiling, quad.y *tiling );



	vec2 vcord = vec2(texCoords.x, texCoords.y);

	if(oreOverlayE > 0){
		oreOverlayTexCoords.xy = vec2(vcord.x + 0.0625 * mod((oreOverlayE-1.0), 16.0), vcord.y + 0.0625 * (oreOverlayStartingRow + floor((oreOverlayE-1.0) * 0.0625))); //floor(oreOverlayE/16);
	}else{
		oreOverlayTexCoords.xy = vec2(0.9);
	}
	if(hitPointsE > 0){
		tTexCoords.xy = vec2(vcord.x + 0.0625 * (hitPointsE - 1.0), vcord.y); //hit ovelays are in the first row
	}else{
		tTexCoords.xy = vec2(0.9);
	}


	tTexCoords.zw = vec2( texCoords.x + tiling * xIndex, texCoords.y + tiling * yIndex);

	#IFDEF threeComp
		vertexPos.x += ((xPos - 128.0)+shift.x) * 16.0;
		vertexPos.y += ((yPos - 128.0)+shift.y) * 16.0;
		vertexPos.z += ((zPos - 128.0)+shift.z) * 16.0;
	#ENDIF


	oreOverlayTexCoords.xy += adip;
	tTexCoords.xy += adip;
	tTexCoords.zw += adip;

	//note: this is a directional light!
	lightDir = normalize(gl_LightSource[0].position.xyz);

	vPos = gl_ModelViewMatrix * vec4(vertexPos,1.0);
	normalVec = gl_NormalMatrix * normal;
	camPos = v_inv[3].xyz; // MOD ADDED
	viewDirection = normalize(v_inv[3].xyz - vPos.xyz);

	//handle the special transparent texture (dispaly in build mode)
	if(allLight > 0 && abs(477.0 - (typeE + (layerE * 256.0))) < 0.00001 ){
		extraAlphaFromVS = 0.5f;
	}else{
		extraAlphaFromVS = 0;
	}

	#IFDEF vertexLighting
		vertexLight = vertexLightFunc(normalVec, lightDir, vPos.xyz,  viewDirection, vec4(occlusion.xyz ,1.0), occlusion.w);
	#ENDIF

	gl_Position =  gl_ProjectionMatrix * vPos;


}
