#version 120

#IFDEF shadow
#extension GL_EXT_gpu_shader4: enable
#ELSEIF texarray
#extension GL_EXT_gpu_shader4: enable
#ELSEIF normaltexarray
#extension GL_EXT_gpu_shader4: enable
#ENDIF

#IMPORT data/shader/cube/cubeTextures.glsl


uniform float density;
uniform float selectTime;
uniform float extraAlpha;


#IMPORT data/shader/cube/cubeLightVars.glsl

#IFDEF vertexLighting
varying vec4 vertexLight;
#ENDIF

varying float extraAlphaFromVS;
varying vec2 oreOverlayTexCoords;
varying vec4 tTexCoords;
varying vec4 occlusion;
varying float layer;
varying float noLight;
varying vec3 camPos; // MOD ADDED

const float LOG2 = 1.442695;



#IMPORT data/shader/shadow.glsl

vec3 gamma(vec3 color){
    return pow(color, vec3(1.0/2.0));
}



#IMPORT data/shader/cube/cubeLight.glsl

// MOD ADDED BEGIN
const float texturePackResolution = 128.0; // the default resolution
const float textureAtlasCount = 16.0;
const float singleTextureSize = 1.0 / textureAtlasCount;
const float texelSize = singleTextureSize / texturePackResolution;

vec4 N(vec2 uv) // isolate normal sampling in a way that allows height in alpha to be added to the texture in the future without rewriting everything.
{
	vec4 bump;
	#IFDEF normaltexarray
		bump = texture2DArray(cTexNormal, vec3(uv, layer));
	#ELSEIF normalmap
		// FIXME use switch(int(layer)) { case 0: break; }
		if(layer <= 0.0){
			bump = texture2D(normalTex0, uv);
		}else if(layer <= 1.5){
			bump = texture2D(normalTex1, uv);
		}else if(layer <= 2.5){
			bump = texture2D(normalTex2, uv);
		}else{
			bump = texture2D(normalTex7, uv);
		}
	#ENDIF
	return vec4(/*normalize*/(bump.xyz * 2.0 - 1.0), bump.w);
}

vec2 AtlasOfs(vec2 uv, vec2 ofs, vec2 uvorg)
{  // clamped to the atlas cell
	return clamp(uv + ofs, uvorg, uvorg + vec2(singleTextureSize));
}

void main()
{
	// MOD ADDED BEGIN
	vec3 dpx = dFdx(vPos.xyz);
	vec3 dpy = dFdy(vPos.xyz);
	vec2 uv = tTexCoords.zw;
	vec2 dtx = dFdx(uv);
	vec2 dty = dFdy(uv);
	vec3 n = normalize(normalVec).xyz;
	vec3 t = normalize(dpy * dtx.y - dpx * dty.y).xyz;
	t = normalize(t - dot(t, n) * n);
	vec3 b = cross(t, n);
	mat3 tbn = mat3(t, b, n);
	vec3 directionToEye = normalize(vec3(-camPos.x, camPos.y, -camPos.z) - vPos.xyz);
	vec2 e = (directionToEye * tbn).xy;
	vec2 atlas = floor(uv * textureAtlasCount) * singleTextureSize; // find atlas cell origin

	float le2 = dot(e,e);
	if (le2 > 1e-7) { // skip if looking straight into texture, don't bother, would divide by zero anyway.
		float le = sqrt(le2);
		vec2 en = e / le; // normalize
		vec2 ed = en * texelSize; // divide out step size
		// see http://en.wikipedia.org/wiki/Finite_difference
		//vec3 dnde = (N(AtlasOfs(uv,ed,atlas)) - N(uv)).xyz; // 1st order
		vec3 dnde = (-N(AtlasOfs(uv,ed*2,atlas)) + 4.0*N(AtlasOfs(uv,ed,atlas)) - 3*N(uv)).xyz/2; // 2nd order -- is it worth it since it's sampling a linear filtered texture?  may need to increase the kernel dimension & compensate on divisor
		float h = -dot(dnde.xy, en);
		float displacementMapScale = .01;
		uv = AtlasOfs(uv, e * (h * displacementMapScale), atlas);
	}
	// MOD ADDED END

	vec4 tex;

	#IFDEF normaltexarray
		tex = texture2DArray(cTex, vec3(uv, layer));
		vec3 bump = texture2DArray(cTexNormal, vec3(uv, layer)).xyz; // N(uv).xyz;
		bump = normalize(bump * 2.0 - 1.0);
	#ELSEIF texarray
		tex = texture2DArray(cTex, vec3(uv, layer));
	#ELSEIF normalmap // vec3 bump = N(uv).xyz;
		vec3 bump;
		if(layer == 0.0){
			tex = texture2D(mainTex0, uv);
			bump = texture2D(normalTex0, uv).xyz;
		}else if(layer <= 1.5){
			tex = texture2D(mainTex1, uv);
			bump = texture2D(normalTex1, uv).xyz;
		}else if(layer <= 2.5){
			tex = texture2D(mainTex2, uv);
			bump = texture2D(normalTex2, uv).xyz;
		}else{
			tex = texture2D(mainTex7, uv);
			bump = texture2D(normalTex7, uv).xyz;
		}
		bump = normalize(bump * 2.0 - 1.0);
	#ELSE
		if(layer == 0.0){
			tex = texture2D(mainTex0, uv);
		}else if(layer <= 1.5){
			tex = texture2D(mainTex1, uv);
		}else if(layer <= 2.5){
			tex = texture2D(mainTex2, uv);
		}else{
			tex = texture2D(mainTex7, uv);
		}
	#ENDIF

	float alphMod = (extraAlpha * (extraAlphaFromVS + tex.a));

	if(alphMod < 0.11){
		discard;
	}

	vec4 oTex;
	vec4 oreTex;


	#IFDEF texarray
		oTex = texture2DArray(cTex, vec3(tTexCoords.xy, 3.0));
		oreTex = texture2DArray(cTex, vec3(oreOverlayTexCoords.xy, 3.0));
	#ELSE
		oTex = texture2D(overlayTex, tTexCoords.xy);
		oreTex = texture2D(overlayTex, oreOverlayTexCoords.xy);
	#ENDIF




	vec4 mixTexOre = mix(tex, oreTex, oreTex.a);
	vec4 mixTex = mix(mixTexOre, oTex, oTex.a);

	vec4 lightedColor;

	#IFDEF lightall
	if(noLight > 0.0001){
		lightedColor = vec4(mixTex.xyz, 1.0);
	}else{
	#ENDIF

	#IFDEF vertexLighting
		lightedColor = vertexLight * mixTex;
	#ELSEIF normaltexarray
		lightedColor = vec4(calculateLight(mixTex, occlusion, uv, bump),1.0);
	#ELSEIF normalmap
		lightedColor = vec4(calculateLight(mixTex, occlusion, uv, bump),1.0);
	#ELSE
		lightedColor = vec4(calculateLight(mixTex, occlusion, uv),1.0);
	#ENDIF

	#IFDEF lightall
	}
	#ENDIF

	lightedColor.a = alphMod;

	lightedColor.rgb += selectTime;

	gl_FragColor = lightedColor;

	#IFDEF shadow
		float shadow_coef = occlusion.w*(1.0-shadowCoef());


		//shadow_coef += occLight;
		gl_FragColor.rgb = gl_FragColor.rgb * min(1.0, ((1.0-shadow_coef)+0.17))*1.2;
	#ENDIF

}
